package edu.nyu.scps.pearl;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PointF;
import android.os.Build;
import android.util.DisplayMetrics;
import android.view.MotionEvent;
import android.view.View;


public class BackgroundView extends View {
    private final int n = 5;	//number of Pearls
    private Pearl[] a = new Pearl[n];
    private PointF touchPoint = new PointF(); //location of touch in dp
    private final float density;              //pixels per dp
    private Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);


    public BackgroundView(Context context) {
        super(context);
        setBackgroundColor(Color.BLACK);
        paint.setColor(Color.WHITE);
        paint.setStrokeWidth(0f);   //hairline

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
            //Turn off hardware acceleration,
            //because it would not call onDraw as often as it should.
            setLayerType(LAYER_TYPE_SOFTWARE, paint);
        }

        Resources resources = getResources();
        DisplayMetrics displayMetrics = resources.getDisplayMetrics();
        density = displayMetrics.density;

        for (int i = 0; i < n; ++i) {
            a[i] = new Pearl();
        }

        setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent motionEvent) {
                //Convert pixels to dp by dividing by density.
                touchPoint.set(motionEvent.getX() / density, motionEvent.getY() / density);
                return true;
            }
        });

        Runnable runnable = new Runnable() {

            //This method is run by a thread that is not the UI thread.
            @Override
            public void run() {
                for (;;) {  //infinite loop
                    for (int i = 0; i < n; ++i) {
                        //The first Pearl is dragged towards the touchPoint.
                        //Each subsequent Pearl is dragged towards its predecessor.
                        PointF p = i == 0 ? touchPoint : a[i-1].getCenter();
                        a[i].dragTowards(p);
                    }

                    postInvalidate();   //Call onDraw.

                    //Sleep for 1/60 of a second.
                    try {
                        Thread.sleep(1000L / 60L);   //milliseconds
                    } catch (InterruptedException interruptedException) {
                    }
                }
            }
        };

        Thread thread = new Thread(runnable);   //a thread that is not the UI thread
        thread.start();   //The thread will execute the run method of the Runnable object.
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        //Convert pixels to dp by dividing by density.
        touchPoint.set(getWidth() / (2f * density), getHeight() / (2f * density));

        for (int i = 0; i < n; ++i) {
            a[i].setCenter(touchPoint);
        }
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        //Convert dp to pixels by multiplying times density.
        canvas.scale(density, density);

        for (int i = 0; i < n; ++i) {
            //The first Pearl hangs from the touchPoint.
            //Each subsequent Pearl hangs from its predecessor.
            PointF p = i == 0 ? touchPoint : a[i-1].getCenter();
            PointF center = a[i].getCenter();

            paint.setStyle(Paint.Style.FILL);
            canvas.drawCircle(center.x, center.y, 16.0f, paint);

            paint.setStyle(Paint.Style.STROKE);
            canvas.drawLine(center.x, center.y, p.x, p.y, paint);
        }
    }

}
